/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/


/*
 * Overrides abstract grid controller to support entering series of estimated values
 * for an estimate. Override is necessary because the REST API used to get and modify
 * estimation series information does not fit into the pattern expected by EPMGridController.
 * 
 * EPMGridController expects to be able to do a GET of a collection, then individual GETs of
 * each element in that collection -- each of those elements is then put into a row. Likewise,
 * it expects individual POST, PUT and DELETE methods on a row by row basis.
 * 
 * Estimation series, however, requires doing a single GET to get all the data from a series
 * at once. Yet, each of those rows in the series needs to become a row in the table.
 */
(function(){
    dojo.provide("com.ibm.epm.web.core.grid.controller.EPMEstimateGridController");
	
    dojo.require("com.ibm.epm.web.core.grid.controller.EPMGridController");
    dojo.require("com.ibm.epm.web.core.util.internal.Constants");
	dojo.require("com.ibm.epm.web.core.grid.controller.GridModelClient");
        
    var gridModelClient = com.ibm.epm.web.core.grid.controller.GridModelClient;
    var abstractEPMGridController = com.ibm.epm.web.core.grid.controller.EPMGridController;
    var Constants = com.ibm.epm.web.core.util.internal.Constants;
    
    dojo.declare("com.ibm.epm.web.core.grid.controller.EPMEstimateGridController", abstractEPMGridController, {
        loadedElement: null,
		budgetURI: null,   // TODO (is loadedElement enough?)
		
        /* 
         * initializeController calling the super function
         */
        initializeController: function(allElementsURI, viewer, model, linksToBeLoaded) {
            this.inherited(arguments);
        },
        
        /* 
         * postCreate calling the super function
         */
        postCreate: function(){
            this.inherited(arguments);
        },
        
        /* 
         * buildGridModel - Creates the grid model
         */
        buildGridModel: function(){
            this._updateEstimateModel();
        },
        
        /* 
         * massageCodeLinks - This function will wrap the links into an array. Before
         * adding to array it checks whether it is removed or not.
         */
        massageCodeLinks: function(massagedLinks, collectionElementsLinks){
            for (var i in collectionElementsLinks) {
                massagedLinks.push(collectionElementsLinks[i]);
            }
        },
        
        /* 
         * If a loadedElement has been provided, this'll load the series info from that loadedElement
         * into the model. If loadedElement is null, then it'll leave it available for creating new
         * rows in the series.
         * 
         * Instead of doing a GET of the entire collection, this method will use what is already
         * stored in loadedElement to determine what rows to add.
         */
        _updateEstimateModel: function() {
			console.log("REACHED _updateEstimateModel");
			
            var self = this;
            var codeLinks = null;
            
			// No data to put in model (assuming new yet-to-be-saved resource being created)
            if (self.loadedElement == null) {
                self._createBlankRow();
                return;
            }
            
			// Pull out all the series data, creating one row for each estimated value
			if (this._allElementsURI == "Estimation") {
                var references = self.loadedElement.series;
				codeLinks = new Array(1);
				for (var k in references) {										
					var itemHandle = {
						title: references[k].id,
						relativeHref: references[k].id
					};
					codeLinks.push(itemHandle);
				}
            }
            // Convert the links to an array and update the model
            if (codeLinks) {
                var massagedLinks = new Array(1);
                this.massageCodeLinks(massagedLinks, codeLinks);
                self.updateModel(false, massagedLinks);
            }
			
			// No data to put in model (estimate exists but has no rows)
            else {
                self._createBlankRow();
            }
        },
		
		/**
		 * This method will be called from gridController when a new element is inserted
		 * overriding the method so that references are handled separately.
		 * 
		 * Instead of doing one GET for every row, this method will pull that info out
		 * of the correct array element in the series stored in loadedElement.
		 * 
		 * @param {Object} rowURI
		 * @param {Object} rowID
		 * @param {Object} linkName
		 */
		_getAndUpdateRow: function(rowURI, rowID, linkName) {
			console.log("REACHED _getAndUpdateRow, rowID: " + rowID);
			
			if (this._allElementsURI != "Estimation") {
				this.inherited(arguments);
			}
			
			else 
			{
				if (this.loadedElement !== null) {
					var found = false;
					var row = {
						name : "",
		                low: 0.0,
		                mostLikely: 0.0,
				        high: 0.0,
						timeOffset: 0
						};
						
					var references = this.loadedElement.series;
					for (var k in references) {
						if ((references[k].id) == rowURI) {
							row.name = references[k].name;
							row.low =  references[k].low;
							row.mostLikely =  references[k].mostLikely;
							row.high =  references[k].high;
							row.timeOffset =  references[k].timeOffset;
							found = true;
							break;
						}
					}
					
					this._updateGridRow(row, rowID, linkName);
					this._getLinkedResources(row, rowID);
				}					
			}
        },
		
// TODO doing save new (in one post) first... then support updating existing record
        /*
         * Persist the entire estimate.  How this is done depends on whether this is saving a new
         * estimate, or is updating an existing estimate.  For a new estimate, the entire
         * save is done by a single post. Updates are more complex and require a post of each
         * row that has changes.  Plus, some rows may be marked for deletion.
         */
		persistGrid: function() {
			return;
			
			console.log("Controller's persistGrid: uri is " + this._allElementsURI);
			
			if (this._allElementsURI != "Estimation") {
				this.inherited(arguments);
			}
			
			else {
				var isNew = true; // TODO figure out how to tell
				
				// The series of estimated values is the most complex... handle it in subfunction
				var seriesArray = new Array();
				var todeleteRowIDs = this._prepareSave(seriesArray);
				
				var params = {
					units: [],
					metric: [],
					startDate: [],
					series: []
				};

				if (seriesArray && (seriesArray.length > 0)) {
					params.series = seriesArray;
				}
				
				var self = this;
				var persistRowFunctions = {
					_persistRowSuccess: function(savedElement){
						// self._refreshBudgets(todeleteRowIDs);   TODO WHAT IS THIS?
					},
					
					_persistRowFailure: function(errorMsg){
						console.log(errorMsg);
						self._propogateErrorMessage(errorMsg);  // TODO
					}
				};
				
				var srh = new ServiceResponseHandler(persistRowFunctions, "_persistRowSuccess", "_persistRowFailure");
				if (this.budgetURI != null) {
					gridModelClient.postURI(srh, this.budgetURI, params);
				}
			}                     
        },
		
		
// TODO
		_prepareSave: function(financialDetailsArray) {
			console.log("Controller's _prepareSave");

			var allRowIds = new Array();
			var allNewRowIds = new Array();
			for (var i = 0; i < this._gridData.getRowCount(); i++) {
                if (this._gridData.getRow) {
					var row = this._gridData.getRow(i);
					if ((row != null) && (row.id != Constants.TOTAL_COST_FIELD) && (row.id != Constants.MARGIN_FIELD)) {
						allRowIds.push(row.id);
						var rowparams = "forecast=" + row.forecast + 
						   ", estimate=" + row.estimate +
						   ", actual=" + row.actual;					   
						
						if((typeof(row.loadedelement[0].chargeCodeUri) != Constants.UNDEFINED) && 
						   (row.loadedelement[0].chargeCodeUri != "") &&						   	
					       (row.chargeCodeUri != row.loadedelement[0].chargeCodeUri))
					    {
					   		var param = row.loadedelement[0].chargeCodeUri + Constants.COLON + Constants.DEL_OPERATION;
							rowparams = rowparams + ", AccountHead=" + param;
							rowparams = rowparams + ", AccountHead=" + row.chargeCodeUri;
					    }
						else if (((typeof(row.loadedelement[0].chargeCodeUri) == Constants.UNDEFINED) || (row.loadedelement[0].chargeCodeUri == "") ) &&
						(typeof(row.chargeCodeUri) != Constants.UNDEFINED) &&
						(row.chargeCodeUri != "")) {
							rowparams = rowparams + ", AccountHead=" + row.chargeCodeUri;
						}	
						  
						if (row.resourceType == Constants.FINANCIAL_DETAILS) {
							if (row.id.match("_unsaved")) {
								allNewRowIds.push(row.id);
								financialDetailsArray.push(rowparams);
							}
							else {
								financialDetailsArray.push((rowparams + ", id=" + row.id));
							}
						}
						else if (row.resourceType == Constants.FINANCIAL_DERIVED) {
							var rowparams = "forecast=" + row.forecast;
							financialDerivedArray.push((rowparams + ", id=" + row.id));
						}
					}
				}					
            }
			
			var references = this.loadedElement.financialDetails;
			for (var j in references) {
				var refId = references[j].id;
				if (!this._containId(allRowIds, refId)) {
					var params = "id=" + refId + Constants.COLON + Constants.DEL_OPERATION ;   		///Send the deleted reference params
					financialDetailsArray.push(params);
				}
			}
			//Delete total so that it will recalculate
			allNewRowIds.push(Constants.TOTAL_COST_FIELD);
			allNewRowIds.push(Constants.MARGIN_FIELD);
			return allNewRowIds;			
		},
		
		/**
		 * _updateFinancialDetails - method will set the charge code uri in the loadedelement
		 * so that second time pressing on save after changing the charge code it throws error
		 */
		_updateFinancialDetails: function(){
			for (var i = 0; i < this._gridData.getRowCount(); i++) {
                if (this._gridData.getRow) {
					var row = this._gridData.getRow(i);
					if ((row != null) && (row.id != Constants.TOTAL_COST_FIELD) && 
					    (row.id != Constants.MARGIN_FIELD) && 
						(row.resourceType == Constants.FINANCIAL_DETAILS)) {
							row.loadedelement[0].chargeCodeUri = row.chargeCodeUri;
					}
				}					
            }
		},
		
		_containId:function(arrayObject, key){
			var found = false;
			if((arrayObject != null) && (arrayObject.length > 0)){
				var num = arrayObject.length;
	            for (var i = 0; i < num; i++) {	            
	                if (arrayObject[i] === key) {
	                    found = true;
	                    break;
	                }
	            }	
			}
			return found;
		},
		
		/**
		 * _refreshBudgets - method will be called soon after saving the budget resource
		 * it will refresh the grid with the latest values
		 * @param {Object} todeleteRowIDs
		 */
		_refreshBudgets: function(todeleteRowIDs){
            this._updateFinancialDetails();
			var self = this;
            var generalFunctions = {
                success: function(loadedElement){
                    if (loadedElement) {
						self.loadedElement = loadedElement;	
						for (var k in todeleteRowIDs){
							self._removeGridRow(todeleteRowIDs[k]);
						}				
						self._updateFinancialModel();
                    }
                },
                failure: function(errorMsg){
                    console.log(errorMsg);
					self._propogateErrorMessage(errorMsg);
                }
            };			            
            var srh = new ServiceResponseHandler(generalFunctions, "success", "failure");
            if (this.budgetURI != null) {
				gridModelClient.getURI(srh, this.budgetURI);
			}            
        },

		_insertFinancials: function(itemHandle){	
			var financials = new Array();		
			for (var i = 0; i < this._gridData.getRowCount(); i++) {
				if (this._gridData.getRow) {
					var financial = this._gridData.getRow(i);
					if (financial != null) {						
						var item = {
							title: financial.name,
							relativeHref: financial.id
						};
						financials.push(item);
						if (financial.name == Constants.TOTAL_COST_FIELD) {
							financials.push(itemHandle);
						}
					}
				}
			}
			this.updateModel(true, financials);			
		}
    });
})();
